#!/usr/bin/perl
#
# Copyright (c) 2005 IDEALX
#
# <pallix@idealx.com>
#
# This software is under GNU PUBLIC LICENSE VERSION 2.
#

use Getopt::Long;
use File::Basename;
use File::Copy;
use strict;
use Cwd;

sub usage {
    print basename($0);
    print " : Generate cache files for source directories\n\n";
    print "Available options:\n\t";
    print "--makefile <file> (path to the makefile that generate a cache",
        "file)\n\t";
    print "[--basedir <name>] (don't assume that rpms are in suse/",
        "{i586,i686,noarch} but in prefix/{i586,i686,noarch}\n\t";
    print "[--srcfile <file>] (the file containing the list of source ";
    print "directories)\n\t";
    print "[--src <path>] (the source directory)\n\t";
    print "(--srcfile or --src is mandatory)\n\t";
    print "[--rule <rule>] (run the rule <rule> for the Makefile ",
        "instead of the default rule all)\n\t";
    print "\n";
    exit(1);
}

my $help = 0;
my @srcfiles = ();
my $makefile = "";
my @srcs = ();
my $basedir = "";
my $rule = "all";

GetOptions("h|help" => \$help, 
           "makefile=s" => \$makefile,
           "src=s" => \@srcs,
           "srcfile=s" => \@srcfiles,
           "basedir=s" => \$basedir,
           "rule=s" => \$rule,
           );
if ($help 
    || $makefile eq ""
    || (@srcfiles == () && @srcs == ())
   ) {
    usage();
}

my @srcdirs = ();
for my $srcfile (@srcfiles) {
        open(FILE, $srcfile) or die "Unable to open '$srcfile' : $!\n";

        while (<FILE>) {
            next if /^#/;
            chomp;
            push(@srcdirs, $_);
        }
        close(FILE);
}

if ($basedir eq "") {
    $basedir = "suse";
}

my $rep;
for my $srcdir (@srcdirs, @srcs) { 
    if (!-d $srcdir) {
        die "FATAL ERROR : $rep doesn't exist. Check your srcfiles.\n";
    }
    for my $arch ("i586", "i686", "noarch") {
        $rep = "$srcdir/$basedir/$arch";
        if (-d $rep) {
            if (!-f "$rep/Makefile") {
                print "Copying Makefile into $rep\n";
                if (copy($makefile, "$rep/Makefile") == 0) { # error
                    die "Error : $!\n";
                }
            }

            print " * Running rule $rule for $rep/Makefile...\n";
            print "="x79,"\n";
            open(PROCESS, "make -C $rep $rule|");
            while(<PROCESS>) {
                print "Info: $_";
            }
            close(PROCESS);
            print "="x79,"\n";
        } else {
            print "! $rep doesn't exists !\n";
        }
    }
}
