#!/usr/bin/perl
#
# Copyright (c) 2005 IDEALX
#
# <pallix@idealx.com>
#
# This software is under GNU PUBLIC LICENSE VERSION 2.

use Getopt::Long;
use File::Basename;
use File::Copy;
use strict;
use Cwd;
use Digest::MD5 qw(md5_base64);

sub usage {
    print basename($0);
    print " : Generate information needed by YaST for source directories\n\n",
        "PLEASE DO NOT GENERATE INFORMATION for local copies of YaST ",
            "CD-ROMs since";
    print " they already have theirs own.\n\n";
    print "Assumption :\n";
    print " - The future source directories have the following structure :
dir_name
    \\--suse
         \\--i586
         \\--i686
         \\--noarch
\n";

    print "Available options:\n\t";
    print "--extraprov <file> (the EXTRA_PROV file, available ",
        "on the first ",
            "CD, in the directory suse/setup/descr)\n\t";
    print "--skelpath <path> (path to the skeleton iso tree)\n\t";
    print "--selpath <path> (path to the .sel files)\n\t";
    print "--cache <path> (path to the cache directory)\n\t";
    print "[--srcfile <file>] (the file containing the list of future source ",
        "directories)\n\t";
    print "[--src <path>] (the future source directory)\n\t";
    print "(--srcfile or --src is mandatory)\n\t";
    print "[-h, --help] (this help text)\n\t";
    print "\n";
    exit(1);
}
## cache functions

sub get_dir_id {
    my ($dir) = @_;
    my $serialized_dir = `ls -1R $dir`;
    my $id = md5_base64($serialized_dir);
    $id =~ s/\//_/g; # no / because we use this id to create a directory
    return $id;
}

sub is_cached {
    my ($dir, $cache_dir) = @_;
    my @files = `ls -1 $cache_dir`;
    my $dir_id = get_dir_id($dir);
    for my $file (@files) {
        chomp($file);
        if ($file eq $dir_id) {
            return $dir_id;
        }
    }
    return undef;
}

sub gen_cache {
    my ($dir, $cache_dir, $extraprov) = @_;
    my $cache_id = get_dir_id($dir);
    my $out_dir = "$cache_dir/$cache_id";
    my $cmd = "create_package_descr -l english -l french -d $dir -o ".
        "$out_dir -x $extraprov \n";
    system($cmd);
    return $cache_id;
}



sub copy_cache_dir {
    my ($cache_dir, $cache_id, $dest_dir) = @_;
    my $cmd = "cp -r $cache_dir/$cache_id/* $dest_dir";
    print $cmd."\n";
    system($cmd);
}

## /cache functions

my $help = 0;
my $srcfile = "";
my @srcs = ();
my $extraprov = "";
my $skelpath = "";
my $selpath = "";
my $cache = "";

GetOptions("h|help" => \$help, 
           "srcfile=s" => \$srcfile, 
           "extraprov=s" => \$extraprov, 
           "src=s" => \@srcs, 
           "skelpath=s" => \$skelpath,
           "selpath=s" => \$selpath,
           "cache=s" => \$cache
          );

if ($help 
    || ($srcfile eq "" && @srcs == () )
    || $extraprov eq ""
    || $skelpath eq "" 
    || $selpath eq ""
    || $cache eq ""
   ) { 
    usage(); 
}

my @srcdirs = ();

# load src files and extract directories names
if ($srcfile ne "") {
    local *FILE;
    open(FILE, $srcfile) or die "Unable to open $srcfile : $!\n";
    while (<FILE>) {
        next if /^#/;
        chomp($_);
        push(@srcdirs, $_);
    }
    close(FILE);
}

my $curdir;
my $cmd;
my $extraprovfile = $extraprov;
# get basename
$extraprovfile =~ s/(.+\/)*(\w+)/$2/;

for my $dir (@srcdirs, @srcs) {
    print "="x40,"\n";
    print "Processing $dir\n";
    print "="x40,"\n";
    # Copy all the needed stuffs :
    #         content
    #     \\--media.1
    #         \\--media
    #         \\--products
    #         \\--directory.yast
    #         \\--info.txt
    system("cp -r $skelpath/* $dir");
    if (-d "$dir/suse/setup") {
        system("rm -rf $dir/suse/setup/");
    }
    my $cache_id = undef;
    if ($cache_id = is_cached("$dir/suse", $cache)) {
        print "Ok we have an entry in the cache for $dir\n";
    } else {
        print "No entry in the cache for $dir\n";
        print "Creating entry...\n";
        $cache_id = gen_cache("$dir/suse", $cache, $extraprov);
      
    }
    
    print "Copying entry files...\n";
    mkdir("$dir/suse/setup") or die "Can't create directory : $!\n";
    mkdir("$dir/suse/setup/descr") or die "Can't create directory : $!\n";
    
    copy_cache_dir($cache, $cache_id, "$dir/suse/setup/descr");

    print "Copying EXTRA_PROV file...\n";

    if (copy($extraprov, "$dir/suse/setup/descr/$extraprovfile") == 0) {
        die "FATAL ERROR : Unable to copy $extraprov : $!\n";
    }

    print "Copying selection files...\n";
    $cmd = "cp -r $selpath/* $dir/suse/setup/descr";
    system($cmd);

    $curdir = cwd();
    chdir("$dir/suse/setup/descr");
    print "Creating file directory.yast...\n";
    system("ls > directory.yast");
    print "Creating file MD5SUMS...\n";
    system("md5sum * > MD5SUMS");
    chdir($curdir);
}
