#!/usr/bin/perl
#
# Copyright (c) 2005 IDEALX
#
# <pallix@idealx.com>
#
# This software is under GNU PUBLIC LICENSE VERSION 2.


use Getopt::Long;
use File::Basename;
use strict;

sub usage {
    print basename($0);
    print " : Create the SuSE tree for generating an ISO image\n\n";
    print "Available options:\n\t";
    print "--comps <path>\n\t";
    print "--srcfile <file> (the file(s) containing the list of source ",
        "directories)\n\t";
    print "--destpath <path> (the path for generating the tree)\n\t";
    print "--skelpath <path> (the path to the skeleton iso tree)\n\t";
    print "--skelsrcpath <path> (path to the skeleton source tree)\n\t";
    print "--selpath <path> (path to the .sel files)\n\t";
    print "--extraprov <file> (path to the EXTRA_PROV file, available ",
        "on the first ",
        "CD, in the directory suse/setup/descr)\n\t";
    print "--autofile <file> (autoyast XML file)\n\t";
    print "--prescripts <path> (path to the pre-scripts directory)\n\t";
    print "--postscripts <path> (path to the post-scripts directory)\n\t";
    print "--cache <path> (path to the cache directory)\n\t";
    print "--bindir <path> (path to the 'binaries' directory)\n\t";
    print "[--verbose (print more information)]\n\t";
    print "[-h, --help] (this help text)\n\t\n\n";
    print "NOTE : We assume that all path in srcfiles containing the ",
        "word *update* have the following ",
        "structure : rpm/{i586,i686,noarch} instead of ",
            "suse/{i586,i686,noarch}\n";
    print "\n";
    exit(1);
}

my $help = 0;
my $verbose = 0;
my $srcfile = "";
my $skelpath = "";
my $destpath = "";
my $comps = "";
my $autofile = "";
my $skelsrcpath = "";
my $extraprov = "";
my $selpath = "";
my $prescripts = "";
my $postscripts = "";
my @srcfiles = ();
my $cache = "";
my $bindir = "";

GetOptions("h|help" => \$help, 
           "verbose" => \$verbose,
           "srcfile=s" => \@srcfiles, 
           "skelpath=s" => \$skelpath,
           "destpath=s" => \$destpath, 
           "comps=s" => \$comps, 
           "autofile=s" => \$autofile,
           "skelsrcpath=s" => \$skelsrcpath,
           "extraprov=s" => \$extraprov,
           "selpath=s" => \$selpath ,
           "prescripts=s" => \$prescripts,
           "postscripts=s" => \$postscripts,
           "cache=s" => \$cache,
           "bindir=s" => \$bindir,
          );

if ($help
    || @srcfiles == ()
    || $skelpath eq ""
    || $destpath eq ""
    || $comps eq ""
    || $autofile eq ""
    || $skelsrcpath eq ""
    || $extraprov eq ""
    || $selpath eq ""
    || $prescripts eq ""
    || $cache eq ""
    || $bindir eq ""
   ) {
    usage();
}


my $cmd;

if (! -d $destpath) {
    mkdir($destpath) or die "Unable to create directory $destpath : $!\n";
}

my @srcdirs = ();
for $srcfile (@srcfiles) {
    open(FILE, $srcfile) or die "Unable to open $srcfile : $!\n";
    while (<FILE>) {
        next if /^#/;
        chomp;
        push(@srcdirs, $_);
    }
    close(FILE);
}

my $args = "";
my $rep;
my $basedir = "rpm";
# build arguments for gen_rpm_base.pl
for my $srcdir (@srcdirs) { 
    if(! -d $srcdir) { 
	    die "FATAL ERROR : '$srcdir' doesn't exist (check your srcfiles)\n\n";
	}
    for my $arch ("i586", "i686", "noarch") {
        if($rep =~ /update/) {
            $rep = "$srcdir/$basedir/$arch";
        } else {
            $rep = "$srcdir/suse/$arch";
        }
        if (-d $rep) { 
            $args = $args." --source $rep ";
        } else { 
		  print "$rep doesn't exists\n"; 
		}
    }
}

$args = $args." --comps $comps --destpath $destpath --suse";

print "Generating autoyast file from template...\n";
#system("cp $autofile $destpath/autoinst.xml");
$cmd = "perl suse_xml_add_scripts.pl --autofile $autofile --prescripts ".
    "$prescripts --postscripts $postscripts ".
    "--outfile $destpath/autoinst.xml.stage1";

if ($verbose) { print $cmd."\n"; }
system($cmd);

$cmd = "perl suse_comps2xml.pl --comps $comps ".
    "--autofile $destpath/autoinst.xml.stage1 ".
    "--outfile $destpath/autoinst.xml";
print "Modifying AutoYaST file...\n";

if ($verbose) { print $cmd."\n"; }
system($cmd);

##$cmd = "perl suse_xml_add_bin.pl --bindir $bindir ".
##    "--autofile $destpath/autoinst.xml.stage2 ".
##    "--outfile $destpath/autoinst.xml";
$cmd = "cp -r $bindir/ $destpath";
if ($verbose) { print $cmd."\n"; }
system($cmd);


unlink("$destpath/autoinst.xml.stage1");
##unlink("$destpath/autoinst.xml.stage2");

print "Copying skeleton tree...\n";
# make boot tree

for my $rep (`find $skelpath/* -type d`) {
    # get relative path 
    $rep =~ s/$skelpath\/(\S+)\s*$/$1/;
    $rep = "$destpath/$rep";
    if(! -d $rep) { 
        mkdir($rep) or die "Unable to create $rep : $!\n";
    }        
}

# make hardlink
for my $file (`find $skelpath/* -type f`) {
    chomp($file);
    my $oldfile = "$file";
    $file =~ s/$skelpath\/(\S+)/$1/;
    $file = "$destpath/$file";
    link($oldfile, $file);
}

print "Creating tree...\n";

if ($verbose) { print "perl gen_rpm_base.pl $args\n"; }
open(OUT, "perl gen_rpm_base.pl $args|");
while (<OUT>) {
    print "Info : $_";
}
close(OUT);

print "Generating source information...\n";
$cmd =  "perl suse_gensrc.pl --skelpath $skelsrcpath --src $destpath --extraprov $extraprov --cache $cache --selpath $selpath";
if ($verbose) { print $cmd."\n"; }
system($cmd);

print "Copying script directory...\n";

$cmd = "cp -fr $prescripts $destpath";
if ($verbose) { print $cmd."\n"; }
system($cmd);
$cmd = "chmod +rx $destpath/pre-scripts/*";
if ($verbose) { print $cmd."\n"; }
system($cmd);
