#!/usr/bin/perl
#
# Copyright (c) 2005 IDEALX
#
# <pallix@idealx.com>
#
# This software is under GNU PUBLIC LICENSE VERSION 2.

use Getopt::Long;
use File::Basename;
use strict;
use Cwd;

sub usage {
    print basename($0);
    print " : Add pre and post script to an AutoYaST file\n\n";
    print "Available options:\n\t";
    print "--autofile <file> (autoyast XML file)\n\t";
    print "--prescripts <path> (path to the pre-scripts directory)\n\t";
    print "--postscripts <path> (path to the post-scripts directory)\n\t";
    print "--outfile <file> (path to the final AutoYaST file)\n\t";
    print "[--dirname <string>] (the name of the directory in the ISO ",
        "that will contain the pre-scripts, 'pre-scripts' by default)\n\t";
    print "[-h, --help] (this help text)\n\t";
    print "\n";
    exit(1);
}

my $help = 0;
my $prescripts = "";

my $postscripts = "";
my $autofile = "";
my $outfile = "";
my $dirname ="pre-scripts";

GetOptions("h|help" => \$help, 
           "prescripts=s" => \$prescripts,
           "autofile=s" => \$autofile,
           "postscripts=s" => \$postscripts,
           "outfile=s" => \$outfile,
           "dirname=s" => \$dirname,
          );

if ($help
    || $autofile eq ""
    || $outfile eq ""
   ) {
    usage();
}

my $dir;
my @files;
my $xml_post = "";
# read post-scripts and generate appropriate XML
if ($postscripts ne "") {
    opendir(DIR, $postscripts) or 
        die "Unable to open '$postscripts' : $!\n";
    @files = readdir(DIR);
    closedir(DIR);
    for my $file (@files) {
        next if ($file =~ /^\./);
        # get extension
        $file =~ /.*\.(.+)$/;
        if($1 eq "sh" || $1 eq "pl") {
            $xml_post .= "\n<script>\n".
                "<filename>$file</filename>\n".
                "<interpreter>".($1 eq "sh" ? "shell" : "perl").
                "</interpreter>\n".
                "<source><![CDATA[\n";
            open(FILE, "$postscripts/$file") or 
                die "Unable to open '$file': $!\n";
            while(<FILE>) {
                $xml_post .= $_;
            }
            close(FILE);
            $xml_post .= "]]></source>\n</script>\n\n";
        } else {
            print "Skipping $file : unknown extension .$1\n";
        }
    }
}

my $xml_pre = "";
# read pre-scripts and generate appropriate XML
if ($prescripts ne "") {
    opendir(DIR, $prescripts) or 
        die "Unable to open '$prescripts' : $!\n";
    @files = readdir(DIR);
    closedir(DIR);
    for my $file (@files) {
        next if ($file =~ /^\./);
        # get extension
        $file =~ /.*\.(.+)$/;
        if($1 eq "sh" || $1 eq "pl") {
            $xml_pre .= "\n<script>\n".
                "<interpreter>".($1 eq "sh" ? "shell" : "perl").
                "</interpreter>\n".
                "<location>file:///$dirname/$file</location>\n".
                "</script>\n\n";
        } else {
            print "Skipping $file : unknown extension .$1\n";
        }
    }
}

# insert generated XML
open(OUT, ">", $outfile) or die "Unable to open '$outfile' : $!\n";

open(FILE, $autofile) or die "Unable to open '$autofile' : $!\n";
while(<FILE>) {
    if(/<\/post-scripts/) {
        print OUT $xml_post;
    } elsif (/<\/pre-scripts/) {
        print OUT $xml_pre;
    }
    print OUT;
}
close(FILE);
close(OUT);
